@@ -18,17 +18,17 @@
 from fedora.client import (AccountSystem, AppError, ServerError)
 from bugzilla.rhbugzilla import RHBugzilla
 from pkgdb2client import PkgDB, PkgDBException, __version__
+import pkgdb2client
 import argparse
 import logging
-import getpass
 import koji
-import fedora_cert
 
 
 KOJI_HUB = 'http://koji.fedoraproject.org/kojihub'
 RH_BZ_API = 'https://bugzilla.redhat.com/xmlrpc.cgi'
 
-pkgdbclient = PkgDB('https://admin.fedoraproject.org/pkgdb')
+pkgdbclient = PkgDB('https://admin.fedoraproject.org/pkgdb',
+                    login_callback=pkgdb2client.ask_password)
 fasclient = AccountSystem('https://admin.fedoraproject.org/accounts')
 BOLD = "\033[1m"
 RED = "\033[0;31m"
@@ -48,30 +48,6 @@ class ActionError(Exception):
     pass
 
 
-def __do_login(username=None, password=None):
-    ''' Returned a BaseClient with authentification
-
-    If the username is None, tries to retrieve it from fedora_cert.
-
-    :arg pkgdbclient a PackageDB object to which username and password
-    are added
-    :karg username FAS username, if None it is asked to the user
-    :karg password FAS password, if None it is asked to the user
-    '''
-    if pkgdbclient.is_logged_in:
-        return
-    else:
-        if username is None:
-            try:
-                username = fedora_cert.read_user_cert()
-            except:
-                LOG.debug('Could not read Fedora cert, using login name')
-                username = raw_input('FAS username: ')
-        password = getpass.getpass('FAS password: ')
-        pkgdbclient.username = username
-        pkgdbclient.login(username, password)
-
-
 def _get_acls_info(acls):
     ''' Re-order the ACLs as provided by the PkgDB API in a way that can be
     easily printed.
@@ -236,11 +212,6 @@ def setup_parser():
     parser_list = subparsers.add_parser(
         'list',
         help='List package according to the specified criteria')
-    parser_list.add_argument(
-        '--all', action="store_true",
-        default=False, dest='all',
-        help="Query all the package in the collection. This may take a "
-        "while.")
     parser_list.add_argument(
         '--nameonly', action="store_true",
         default=False, dest='name_only',
@@ -255,7 +226,12 @@ def setup_parser():
         help="List all orphaned and eol'd packages")
     parser_list.add_argument(
         '--user', dest='user', default=None,
-        help="List all the packages of the user <user>")
+        help="List all the packages of that user <user> has ACLs on")
+    parser_list.add_argument(
+        '--poc', action="store_true",
+        default=False, dest='poc',
+        help="Used in combination with `--user`, restrict the list to the "
+        "packages the user is the point of contact of")
     parser_list.add_argument(
         '--branch', dest='branch', default=None,
         help="Specify a branch (default:'all')")
@@ -387,7 +363,10 @@ def do_acl(args):
                  'component': args.package})
             print "{0} bugs open (new, assigned, needinfo)".format(len(bugbz))
 
-    for pkg in output['packages']:
+    for pkg in sorted(
+            output['packages'],
+            key=lambda pkg: pkg['collection']['branchname'],
+            reverse=True):
         if pkg['collection']['status'] == 'EOL':
             continue
         owner = pkg['point_of_contact']
@@ -412,23 +391,27 @@ def do_acl(args):
 
         # print ACL information
         print "{0}ACLs:".format(" " * 8)
-        acls = _get_acls_info(pkg['acls'])
-        for user in acls:
-            if user.startswith('group::'):
-                tmp = " " * 3 + user
-            else:
-                tmp = " " * 10 + user
-            tmp = tmp + " " * (24 - len(tmp))
-            for acl_title in ["watchbugzilla", "watchcommits",
-                              "commit", "approveacls"]:
-                #print '\n', acl_title
-                if acl_title in acls[user]:
-                    aclout = acls[user][acl_title]
-                    tmp = tmp + aclout + " " * (16 - len(aclout))
+        if 'acls' in pkg:
+            acls = _get_acls_info(pkg['acls'])
+            users = sorted(acls, key=lambda user: user.replace('group::', ''))
+            for user in users:
+                if user.startswith('group::'):
+                    tmp = " " * 3 + user
                 else:
-                    tmp = tmp + " " * 16
-            if tmp is not None and tmp.strip() != "":
-                print tmp
+                    tmp = " " * 10 + user
+                tmp = tmp + " " * (24 - len(tmp))
+                for acl_title in ["watchbugzilla", "watchcommits",
+                                  "commit", "approveacls"]:
+                    #print '\n', acl_title
+                    if acl_title in acls[user]:
+                        aclout = acls[user][acl_title]
+                        tmp = tmp + aclout + " " * (16 - len(aclout))
+                    else:
+                        tmp = tmp + " " * 16
+                if tmp is not None and tmp.strip() != "":
+                    print tmp
+        else:
+            print '           No ACLs found'
 
         # print the last build
         if args.extra:
@@ -441,27 +424,51 @@ def do_list(args):
 
     '''
     LOG.info("pattern  : {0}".format(args.pattern))
-    LOG.info("all      : {0}".format(args.all))
+    LOG.info("poc      : {0}".format(args.poc))
     LOG.info("orphaned : {0}".format(args.orphaned))
     LOG.info("user     : {0}".format(args.user))
     LOG.info("name only: {0}".format(args.name_only))
     LOG.info("branch   : {0}".format(args.branch))
 
     pattern = args.pattern
-    if not pattern or args.all:
+    if not pattern:
         pattern = '*'
-    elif not pattern and not args.all:
-        raise argparse.ArgumentTypeError("Not enough arguments given")
 
     if not pattern.endswith('*'):
         pattern += '*'
 
-    output = pkgdbclient.get_packages(
-        pattern=pattern,
-        branches=args.branch,
-        poc=args.user,
-        orphaned=args.orphaned
-    )
+    if args.user and not args.poc:
+        version = pkgdbclient.get_version()
+        if version >= (1, 6):
+            output = pkgdbclient.get_packager_package(args.user)
+            output['packages'] = output['point of contact']
+            for pkg in output['co-maintained']:
+                if pkg not in output['packages']:
+                    output['packages'].append(pkg)
+            for pkg in output['watch']:
+                if pkg not in output['packages']:
+                    output['packages'].append(pkg)
+        else:
+            # This is for backward compat but it's way slower
+            output = pkgdbclient.get_packager_acls(
+                packagername=args.user,
+                page='all',
+            )
+            output2 = {'packages': []}
+            for item in output['acls']:
+                pkg = item['packagelist']['package']
+                if pkg not in output2['packages']:
+                    output2['packages'].append(pkg)
+            output = output2
+    else:
+        output = pkgdbclient.get_packages(
+            pattern=pattern,
+            branches=args.branch,
+            poc=args.user,
+            orphaned=args.orphaned,
+            page='all',
+        )
+
     cnt = 0
     for pkg in sorted(output['packages'], key=lambda pkg: (pkg['name'])):
         out = "   " + pkg['name'] + ' ' * (33 - len(pkg['name'])) + \
@@ -494,7 +501,7 @@ def do_orphan(args):
     else:
         branches = [args.branch]
 
-    __do_login(args.username)
+    pkgdbclient.username = args.username
 
     output = pkgdbclient.orphan_packages(pkgs, branches)
     for msg in output.get('messages', []):
@@ -523,7 +530,7 @@ def do_unorphan(args):
     else:
         branches = [args.branch]
 
-    __do_login(args.username)
+    pkgdbclient.username = args.username
 
     username = args.poc or args.username or pkgdbclient.username
     LOG.info("new poc : {0}".format(username))
@@ -558,7 +565,7 @@ def do_request(args):
     if args.cancel:
         status = 'Obsolete'
 
-    __do_login(args.username)
+    pkgdbclient.username = args.username
     LOG.info("user    : {0}".format(pkgdbclient.username))
 
     output = pkgdbclient.update_acl(
@@ -600,7 +607,7 @@ def do_update(args):
     if args.approve:
         status = "Approved"
 
-    __do_login(args.username)
+    pkgdbclient.username = args.username
 
     output = pkgdbclient.update_acl(
         args.package,
@@ -664,10 +671,16 @@ def main():
             insecure=True)
         pkgdbclient = PkgDB(
             'https://admin.stg.fedoraproject.org/pkgdb',
+            login_callback=pkgdb2client.ask_password,
             insecure=True)
 
     return_code = 0
 
+    if arg.password:
+        pkgdbclient.password = arg.password
+    if arg.username:
+        pkgdbclient.username = arg.username
+
     try:
         arg.func(arg)
     except KeyboardInterrupt:
