@@ -15,15 +15,24 @@
 # license.
 """
 
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
+import getpass
 import logging
+import os
+import pkg_resources
 
+import fedora_cert
 from fedora.client import AuthError
 import requests
 
+
 logging.basicConfig()
 LOG = logging.getLogger("pkgdb2client")
 
-__version__ = '2.2'
+__version__ = pkg_resources.get_distribution('packagedb-cli').version
 PKGDB_URL = r'https://admin.fedoraproject.org/pkgdb/'
 
 
@@ -60,13 +69,36 @@ def _parse_service_form(response):
     return (parsed.form.attrs['action'], inputs)
 
 
+def ask_password(username=None, bad_password=False):
+    """ Example login_callback to ask username/password from user
+    :kwarg username: Username
+    :type username: str
+    :kwarg bad_password: Whether there was a previous failed login attempt
+    :type bad_password: bool
+    :return: username, password
+    :rtype: tuple
+    """
+    if bad_password:
+        print "Bad password, please retry"
+    if not username:
+        try:
+            username = fedora_cert.read_user_cert()
+        except fedora_cert.fedora_cert_error:
+            LOG.debug('Could not read Fedora cert, asking for username')
+            username = raw_input("Username: ")
+    password = getpass.getpass("FAS password for user {0}: ".format(username))
+    return username, password
+
+
 class PkgDB(object):
     ''' PkgDB class used to interact with the Package DB instance via its
     API.
 
     '''
 
-    def __init__(self, url=PKGDB_URL, insecure=False):
+    def __init__(self, url=PKGDB_URL, insecure=False, cookies=None,
+                 login_callback=None, login_attempts=3,
+                 sessionfile="~/.cache/pkgdb-session.pickle"):
         ''' Constructor for the PkgDB object used to query the package
         database.
 
@@ -78,12 +110,32 @@ class PkgDB(object):
             is.  Useful for development but should not be used in
             production code.
         :type insecure: bool
+        :kwarg login_callback: Function to be called to provide username and/or
+            password, it must accept two keyword arguments, `username` and
+            `bad_password`. Username is the currently known username (can be
+            None) and bad_password is a bool to specifies whether a bad
+            password was supplied earlier. It needs to return a (username,
+            password) tuple. See pkgdb2client.ask_password() for an example.
+        :type login_callback: function
+        :kwarg login_attempts: How often a login might fail after
+            login_callback() was called before a API call fails permanently.
+        :type login_attempts: int
 
         '''
         self.url = url
         self.session = requests.session()
         self.insecure = insecure
-        self.__logged_in = False
+        self.username = None
+        self.password = None
+        self.login_callback = login_callback
+        self.login_attempts = login_attempts
+        self.sessionfile = os.path.expanduser(sessionfile)
+
+        try:
+            with open(self.sessionfile, "rb") as sessionfo:
+                self.session.cookies = pickle.load(sessionfo)["cookies"]
+        except (IOError, KeyError, TypeError):
+            pass
 
     def __send_request(self, url, method, params=None, data=None):
         ''' Send a http request to the provided URL with the provided
@@ -104,14 +156,32 @@ class PkgDB(object):
             data=data,
             verify=not self.insecure,
         )
+        self._save_cookies()
         return req
 
+    def _save_cookies(self):
+        try:
+            with open(self.sessionfile, 'rb') as sessionfo:
+                data = pickle.load(sessionfo)
+        except:
+            data = {}
+        try:
+            with open(self.sessionfile, 'wb', 0600) as sessionfo:
+                sessionfo.seek(0)
+                data["cookies"] = self.session.cookies
+                pickle.dump(data, sessionfo)
+        except:
+            pass
+
     @property
     def is_logged_in(self):
         ''' Return whether the user if logged in or not. '''
-        return self.__logged_in
 
-    def login(self, username, password, openid_insecure=False):
+        response = self.session.get(self.url + '/login/')
+        return "logged in as" in response.text
+
+    def login(self, username=None, password=None, openid_insecure=False,
+              response=None):
         ''' Login the user on pkgdb2.
 
         :arg username: the FAS username of the user.
@@ -126,6 +196,17 @@ class PkgDB(object):
             production.
         :type openid_insecure: bool
         '''
+        if not username:
+            username = self.username
+        if not password:
+            password = self.password
+        if self.login_callback and not password:
+            username, password = self.login_callback(username=username,
+                                                     bad_password=False)
+
+        if not username or not password:
+            raise PkgDBAuthException('Username or password missing')
+
         import re
         from urlparse import urlparse, parse_qs
 
@@ -135,7 +216,8 @@ class PkgDB(object):
         motif = re.compile(fedora_openid)
 
         # Log into the service
-        response = self.session.get(self.url + '/login/')
+        if not response:
+            response = self.session.get(self.url + '/login/')
 
         if '<title>OpenID transaction in progress</title>' \
                 in response.text:
@@ -144,6 +226,10 @@ class PkgDB(object):
             if not motif.match(openid_url):
                 raise PkgDBException(
                     'Un-expected openid provider asked: %s' % openid_url)
+        elif 'logged in as' in response.text:
+            # User already logged in via its cookie file by default:
+            # ~/.cache/pkgdb-session.pickle
+            return
         else:
             data = {}
             for resp in response.history:
@@ -177,7 +263,80 @@ class PkgDB(object):
             method='POST',
             data=output['response'])
 
-        self.__logged_in = True
+        return output
+
+    def call_api(self, path, params=None, data=None):
+        ''' call the API.
+
+        :arg path: The path to call
+        :type path: str
+        :arg params: URL params for the API call
+        :type params: dict
+        :arg data: POST data for the API call
+        :type data: dict
+        :return: requests response object
+        :rtype: requests.models.Response
+        :raise PkgDBAuthException: If login is required and fails
+        '''
+        if data:
+            method = "POST"
+        else:
+            method = "GET"
+
+        url = self.url + "/api" + path
+        response = self.__send_request(url=url, method=method, data=data,
+                                       params=params)
+        if '<title>OpenID transaction in progress</title>' \
+                in response.text:
+            bad_password = False
+            password = self.password
+            username = self.username
+            success = False
+            for count in xrange(self.login_attempts):
+                if not username or not password or bad_password:
+                    if self.login_callback:
+                        username, password = self.login_callback(
+                            username=username,
+                            bad_password=bad_password
+                        )
+                    else:
+                        raise PkgDBAuthException('Authentication required')
+                try:
+                    self.login(username=username, password=password,
+                               response=response)
+                    success = True
+                    break
+                except PkgDBException:
+                    response = None
+                    bad_password = True
+            if not success:
+                raise PkgDBAuthException("Too many failed login attempts")
+            response = self.__send_request(url=url, method=method, data=data,
+                                           params=params)
+        return response
+
+    def handle_api_call(self, path, params=None, data=None):
+        ''' call the API.
+
+        :arg path: The path to call
+        :type path: str
+        :arg params: URL params for the API call
+        :type params: dict
+        :arg data: POST data for the API call
+        :type data: dict
+        :return: the json object returned by the API
+        :rtype: dict
+        :raise PkgDBException: if the API call does not return a http code
+            200.
+
+        '''
+
+        response = self.call_api(path, params, data)
+        output = response.json()
+
+        if response.status_code != 200:
+            LOG.debug('full output %s', output)
+            raise PkgDBException(output['error'])
 
         return output
 
@@ -214,9 +373,6 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'clt_name': clt_name,
             'version': version,
@@ -227,18 +383,7 @@ class PkgDB(object):
             'kojiname': kojiname,
         }
 
-        req = self.__send_request(
-            url='{0}/api/collection/new/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/collection/new/', data=args)
 
     def create_package(
             self, pkgname, summary, description, review_url, status,
@@ -277,9 +422,6 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgname': pkgname,
             'summary': summary,
@@ -294,18 +436,7 @@ class PkgDB(object):
         if critpath:
             args['critpath'] = critpath
 
-        req = self.__send_request(
-            url='{0}/api/package/new/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/new/', data=args)
 
     def get_critpath_packages(self, branches=None, **kwargs):
         ''' Return the list of package names in the critical path.
@@ -342,18 +473,7 @@ class PkgDB(object):
             'format': 'json',
         }
 
-        req = self.__send_request(
-            url='{0}/api/critpath/'.format(self.url),
-            method='GET',
-            params=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/critpath/', params=args)
 
     def get_collections(self, pattern='*', clt_status=None):
         ''' Return the list of collections matching the provided criterias.
@@ -375,18 +495,7 @@ class PkgDB(object):
             'clt_status': clt_status,
         }
 
-        req = self.__send_request(
-            url='{0}/api/collections/'.format(self.url),
-            method='GET',
-            params=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/collections/', params=args)
 
     def get_package(self, pkgname, branches=None, eol=False):
         ''' Return the information of a package matching the provided
@@ -401,7 +510,7 @@ class PkgDB(object):
             If True, it will return results for all collections (including
             EOL).
             If False, it will return results only for non-EOL collections.
-        :type page: boolean
+        :type eol: boolean
         :return: the json object returned by the API
         :rtype: dict
         :raise PkgDBException: if the API call does not return a http code
@@ -415,18 +524,7 @@ class PkgDB(object):
         if eol is True:
             args['eol'] = eol
 
-        req = self.__send_request(
-            url='{0}/api/package/'.format(self.url),
-            method='GET',
-            params=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/', params=args)
 
     def get_packager_acls(
             self, packagername, acls=None, eol=False, poc=None,
@@ -449,7 +547,7 @@ class PkgDB(object):
             If True, it will return results for all collections (including
             EOL).
             If False, it will return results only for non-EOL collections.
-        :type page: boolean
+        :type eol: boolean
         :kwarg poc: a boolean specifying whether the results should be
             restricted to ACL for which the provided packager is the point
             of contact or not. Defaults to None.
@@ -491,18 +589,7 @@ class PkgDB(object):
             if poc is not None:
                 args['poc'] = poc
 
-            req = self.__send_request(
-                url='{0}/api/packager/acl/'.format(self.url),
-                method='GET',
-                params=args)
-
-            output = req.json()
-
-            if req.status_code != 200:
-                LOG.debug('full output %s', output)
-                raise PkgDBException(output['error'])
-
-            return output
+            return self.handle_api_call('/packager/acl/', params=args)
 
         if page == 'all':
             page = 0
@@ -538,18 +625,7 @@ class PkgDB(object):
             'packagername': packagername,
         }
 
-        req = self.__send_request(
-            url='{0}/api/packager/stats/'.format(self.url),
-            method='GET',
-            params=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/packager/stats/', params=args)
 
     def get_packagers(self, pattern='*'):
         ''' Return the list of packagers matching the provided criterias.
@@ -571,18 +647,30 @@ class PkgDB(object):
             'pattern': pattern,
         }
 
-        req = self.__send_request(
-            url='{0}/api/packagers/'.format(self.url),
-            method='GET',
-            params=args)
+        return self.handle_api_call('/packagers/', params=args)
 
-        output = req.json()
 
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
+    def get_packager_package(self, packager):
+        ''' Return the list of packages related to the specified packager.
 
-        return output
+        The list of packages is split into three categories:
+            point of contact, co-maintained, watch
+        These are the same three categories used in the packager's page in
+        the pkgdb2 UI.
+
+        :arg packager: The name the packager to query the packages of
+        :type pattern: str
+        :return: the json object returned by the API
+        :rtype: dict
+        :raise PkgDBException: if the API call does not return a http code
+            200.
+
+        '''
+        args = {
+            'packagername': packager,
+        }
+
+        return self.handle_api_call('/packager/package/', params=args)
 
     def get_packages(
             self, pattern='*', branches=None, poc=None, status=None,
@@ -621,9 +709,9 @@ class PkgDB(object):
             If True, it will return results for all collections (including
             EOL).
             If False, it will return results only for non-EOL collections.
-        :type page: boolean
+        :type eol: boolean
         :kwarg page: The page number to retrieve. If page is 0 or lower or
-            equal to ``all`` then all pages are returned. Defaults to 0.
+            equal to ``all`` then all pages are returned. Defaults to 1.
         :type page: int or ``all``
         :kwarg count: A boolean to retrieve the count of ACLs the user has
             instead of the details. If count is True the page argument will
@@ -661,18 +749,7 @@ class PkgDB(object):
             if eol is True:
                 args['eol'] = eol
 
-            req = self.__send_request(
-                url='{0}/api/packages/'.format(self.url),
-                method='GET',
-                params=args)
-
-            output = req.json()
-
-            if req.status_code != 200:
-                LOG.debug('full output %s', output)
-                raise PkgDBException(output['error'])
-
-            return output
+            return self.handle_api_call('/packages/', params=args)
 
         if page == 'all':
             page = 0
@@ -708,26 +785,12 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgnames': pkgnames,
             'branches': branches,
         }
 
-        req = self.__send_request(
-            url='{0}/api/package/orphan/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/orphan/', data=args)
 
     def retire_packages(self, pkgnames, branches):
         ''' Retires the provided list of packages on the provided list of
@@ -746,26 +809,12 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgnames': pkgnames,
             'branches': branches,
         }
 
-        req = self.__send_request(
-            url='{0}/api/package/retire/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/retire/', data=args)
 
     def unorphan_packages(self, pkgnames, branches, poc):
         ''' Un orphan the provided list of packages on the provided list of
@@ -786,27 +835,13 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgnames': pkgnames,
             'branches': branches,
             'poc': poc,
         }
 
-        req = self.__send_request(
-            url='{0}/api/package/unorphan/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/unorphan/', data=args)
 
     def unretire_packages(self, pkgnames, branches):
         ''' Un retires the provided list of packages on the provided list of
@@ -825,26 +860,12 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgnames': pkgnames,
             'branches': branches,
         }
 
-        req = self.__send_request(
-            url='{0}/api/package/unretire/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/unretire/', data=args)
 
     def update_acl(self, pkgname, branches, acls, status, user):
         ''' Update the specified ACLs, on the specified Branches of the
@@ -876,9 +897,6 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgname': pkgname,
             'branches': branches,
@@ -887,18 +905,7 @@ class PkgDB(object):
             'user': user,
         }
 
-        req = self.__send_request(
-            url='{0}/api/package/acl/'.format(self.url),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/package/acl/', data=args)
 
     def update_collection_status(self, branch, clt_status):
         ''' Update the status of the specified collection.
@@ -917,26 +924,13 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'branch': branch,
             'clt_status': clt_status,
         }
 
-        req = self.__send_request(
-            url='{0}/api/collection/{1}/status/'.format(self.url, branch),
-            method='POST',
-            data=args)
-
-        output = req.json()
-
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
-
-        return output
+        return self.handle_api_call('/collection/{0}/status/'.format(branch),
+                                    data=args)
 
     def update_package_poc(self, pkgnames, branches, poc):
         ''' Update the point of contact of the specified packages on the
@@ -958,24 +952,39 @@ class PkgDB(object):
             200.
 
         '''
-        if not self.is_logged_in:
-            raise PkgDBAuthException('Authentication required')
-
         args = {
             'pkgnames': pkgnames,
             'branches': branches,
             'poc': poc,
         }
+        return self.handle_api_call('/package/acl/reassign/', data=args)
 
-        req = self.__send_request(
-            url='{0}/api/package/acl/reassign/'.format(self.url),
-            method='POST',
-            data=args)
+    def get_version(self):
+        ''' Return a tuple of the pkgdb API version.
 
-        output = req.json()
+        :return: the pkgdb API version
+        :rtype: tuple
+        :raise PkgDBException: if the API call does not return a http code
+            200.
 
-        if req.status_code != 200:
-            LOG.debug('full output %s', output)
-            raise PkgDBException(output['error'])
+        Example of data returned
 
-        return output
+        ::
+
+            (1, 6)
+
+        '''
+        version = self.handle_api_call('/version')
+        if not 'version' in version:
+            raise PkgDBException(
+                'No version information could be retrieved')
+        version = version['version']
+        output = []
+        for el in version.split('.'):
+            try:
+                el = int(el)
+            except ValueError:
+                pass
+            output.append(el)
+
+        return tuple(output)
