@@ -7,24 +7,43 @@ import time
 import unittest
 import uuid
 
+from functools import wraps
+
 import fedora_cert
 
 from pkgdb2client import PkgDB, PkgDBException
 
 
 PKGDB_URL = 'http://127.0.0.1:5000'
+AUTH = True
 
-try:
-    USERNAME = fedora_cert.read_user_cert()
-except:
-    USERNAME = raw_input('FAS username: ')
-PASSWORD = getpass.getpass('FAS password: ')
+if AUTH:
+    try:
+        USERNAME = fedora_cert.read_user_cert()
+    except:
+        USERNAME = raw_input('FAS username: ')
+    PASSWORD = getpass.getpass('FAS password: ')
+    if not PASSWORD:
+        AUTH = False
 
 COL_NAME = str(uuid.uuid1())[:30]
 PKG_NAME = str(uuid.uuid1())[:30]
 VERSION = time.mktime(datetime.datetime.utcnow().timetuple())
 
 
+def auth_only(function):
+    """ Decorator to skip tests if AUTH is set to False """
+    @wraps(function)
+    def decorated_function(*args, **kwargs):
+        """ Decorated function, actually does the work. """
+        if AUTH:
+            return function(*args, **kwargs)
+        else:
+            return 'Skipped'
+
+    return decorated_function
+
+
 class TestPkgdDB(unittest.TestCase):
     ''' Un-authenticated pkgdb2 tests. '''
 
@@ -81,22 +100,20 @@ class TestPkgdDB(unittest.TestCase):
             out['packages'][0]['collection']['branchname'], 'f20')
         self.assertEqual(
             out['packages'][0]['package']['name'], 'guake')
+        out = self.pkgdb.get_package('guake', ['f20', 'f19'])
         self.assertEqual(
-            out['packages'][0]['point_of_contact'], 'pingou')
-
-        ## We do not support multi-branches yet
-        #out = self.pkgdb.get_package('guake', ['f20', 'f19'])
-        #self.assertEqual(
-            #sorted(out.keys()),
-            #['output', 'packages'])
-        #self.assertEqual(out['output'], 'ok')
-        #self.assertEqual(len(out['packages']), 2)
-        #self.assertEqual(
-            #out['packages'][0]['collection']['branchname'], 'f20')
-        #self.assertEqual(
-            #out['packages'][0]['package']['name'], 'guake')
-        #self.assertEqual(
-            #out['packages'][0]['point_of_contact'], 'pingou')
+            sorted(out.keys()),
+            ['output', 'packages'])
+        self.assertEqual(out['output'], 'ok')
+        self.assertEqual(len(out['packages']), 2)
+        self.assertEqual(
+            out['packages'][0]['collection']['branchname'], 'f19')
+        self.assertEqual(
+            out['packages'][1]['collection']['branchname'], 'f20')
+        self.assertEqual(
+            out['packages'][0]['package']['name'], 'guake')
+        self.assertEqual(
+            out['packages'][1]['package']['name'], 'guake')
 
     def test_get_packager_acls(self):
         ''' Test the get_packager_acls function. '''
@@ -111,35 +128,35 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(
             sorted(out.keys()),
             ['acls', 'output', 'page', 'page_total'])
-        self.assertEqual(len(out['acls']), 239)
-        self.assertEqual(out['page_total'], 1)
+        self.assertTrue(len(out['acls']) >= 239)
+        self.assertEqual(out['page_total'], 2)
 
         out = self.pkgdb.get_packager_acls('pingou', page=3)
         self.assertEqual(
             sorted(out.keys()),
             ['acls', 'output', 'page', 'page_total'])
-        self.assertEqual(len(out['acls']), 250)
+        self.assertTrue(len(out['acls']) >= 250)
         self.assertEqual(out['page_total'], 5)
 
         out = self.pkgdb.get_packager_acls('pingou', count=True)
         self.assertEqual(
             sorted(out.keys()),
             ['acls_count', 'output', 'page', 'page_total'])
-        self.assertEqual(out['acls_count'], 1043)
+        self.assertTrue(out['acls_count'] >= 1043)
         self.assertEqual(out['page_total'], 1)
 
         out = self.pkgdb.get_packager_acls('pingou', poc=True, count=True)
         self.assertEqual(
             sorted(out.keys()),
             ['acls_count', 'output', 'page', 'page_total'])
-        self.assertEqual(out['acls_count'], 804)
+        self.assertTrue(out['acls_count'] >= 750)
         self.assertEqual(out['page_total'], 1)
 
         out = self.pkgdb.get_packager_acls('pingou', poc=False, count=True)
         self.assertEqual(
             sorted(out.keys()),
             ['acls_count', 'output', 'page', 'page_total'])
-        self.assertEqual(out['acls_count'], 239)
+        self.assertTrue(out['acls_count'] >= 239)
         self.assertEqual(out['page_total'], 1)
 
     def test_get_packager_stats(self):
@@ -151,7 +168,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(
             sorted(out['master'].keys()),
             ['co-maintainer', 'point of contact'])
-        self.assertTrue(out['master']['point of contact'] >= 60)
+        self.assertTrue(out['master']['point of contact'] >= 50)
 
     def test_get_packagers(self):
         ''' Test the get_packagers function. '''
@@ -171,7 +188,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'guacamole-client')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['packages'][1]['name'], 'guacamole-common')
         self.assertEqual(out['page'], 1)
@@ -185,7 +202,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'guacamole-common')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['packages'][1]['name'], 'guacamole-ext')
         self.assertEqual(out['page'], 1)
@@ -199,7 +216,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'guake')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['page'], 1)
         self.assertEqual(out['page_total'], 1)
@@ -212,7 +229,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'guacd')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['page'], 1)
         self.assertEqual(out['page_total'], 1)
@@ -225,7 +242,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'ghex')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['packages'][1]['name'], 'glom')
         self.assertEqual(out['page'], 1)
@@ -252,7 +269,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(out['packages'][0]['name'], 'ghc-parameterized-data')
         self.assertEqual(
             sorted(out['packages'][0].keys()),
-            ['creation_date', 'description', 'name', 'review_url',
+            ['acls', 'creation_date', 'description', 'name', 'review_url',
              'status', 'summary', 'upstream_url'])
         self.assertEqual(out['page'], 2)
         self.assertEqual(out['page_total'], 6)
@@ -261,7 +278,7 @@ class TestPkgdDB(unittest.TestCase):
         self.assertEqual(
             sorted(out.keys()),
             ['output', 'packages', 'page', 'page_total'])
-        self.assertTrue(out['packages'] >= 1351)
+        self.assertTrue(out['packages'] >= 1340)
         self.assertEqual(out['page'], 1)
         self.assertEqual(out['page_total'], 1)
 
@@ -269,6 +286,7 @@ class TestPkgdDB(unittest.TestCase):
 class TestPkgdDBAuth(unittest.TestCase):
     ''' Authenticated pkgdb2 tests. '''
 
+    @auth_only
     def setUp(self):
         """ set up data used in the tests.
         setUp is called before each test function execution.
@@ -276,6 +294,7 @@ class TestPkgdDBAuth(unittest.TestCase):
         self.pkgdb = PkgDB(PKGDB_URL, insecure=True)
         self.pkgdb.login(USERNAME, PASSWORD)
 
+    @auth_only
     def test_1_create_collection(self):
         ''' Test the create_collection function. '''
         out = self.pkgdb.create_collection(
@@ -318,6 +337,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             out['messages'],
             ['Collection "%s" created' % COL_NAME])
 
+    @auth_only
     def test_2_create_package(self):
         ''' Test the create_package function. '''
 
@@ -342,6 +362,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             out['messages'],
             ['Package created'])
 
+    @auth_only
     def test_3_orphan_packages(self):
         ''' Test the orphan_packages function. '''
 
@@ -354,11 +375,12 @@ class TestPkgdDBAuth(unittest.TestCase):
         self.assertEqual(out['output'], 'ok')
         self.assertEqual(
             out['messages'],
-            ['user: pingou changed poc of package: guake from: pingou '
-             'to: orphan on branch: master',
-             'user: pingou changed poc of package: guake from: pingou '
-             'to: orphan on branch: el6'])
+            ['user: pingou changed point of contact of package: guake from: '
+             'pingou to: orphan on branch: master',
+             'user: pingou changed point of contact of package: guake from: '
+             'pingou to: orphan on branch: el6'])
 
+    @auth_only
     def test_4_unorphan_packages(self):
         ''' Test the unorphan_packages function. '''
 
@@ -375,6 +397,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             ['Package guake has been unorphaned on master by pingou',
              'Package guake has been unorphaned on el6 by pingou'])
 
+    @auth_only
     def test_5_retire_packages(self):
         ''' Test the retire_packages function. '''
 
@@ -390,6 +413,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             ['user: pingou updated package: guake status from: Approved to '
              'Retired on branch: master'])
 
+    @auth_only
     def test_6_unretire_packages(self):
         ''' Test the unretire_packages function. '''
 
@@ -405,6 +429,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             ['user: pingou updated package: guake status from: Retired to '
              'Approved on branch: master'])
 
+    @auth_only
     def test_7_update_acl(self):
         ''' Test the update_acl function. '''
 
@@ -419,11 +444,10 @@ class TestPkgdDBAuth(unittest.TestCase):
         self.assertEqual(out['output'], 'ok')
         self.assertEqual(
             out['messages'],
-            ['user: pingou set acl: commit of package: guake from: '
-             'Awaiting Review to: Awaiting Review on branch: master',
-             'user: pingou set acl: commit of package: guake from: '
-             'Awaiting Review to: Awaiting Review on branch: el6'])
+            ['Nothing to update on branch: master for acl: commit',
+             'Nothing to update on branch: el6 for acl: commit'])
 
+    @auth_only
     def test_8_update_collection_status(self):
         ''' Test the update_collection_status function. '''
 
@@ -438,6 +462,7 @@ class TestPkgdDBAuth(unittest.TestCase):
             out['messages'],
             ['Collection updated from "Active" to "EOL"'])
 
+    @auth_only
     def test_9_update_package_poc(self):
         ''' Test the update_package_poc function. '''
 
@@ -451,10 +476,10 @@ class TestPkgdDBAuth(unittest.TestCase):
         self.assertEqual(out['output'], 'ok')
         self.assertEqual(
             out['messages'],
-            ['user: pingou changed poc of package: guake from: orphan to: '
-             'ralph on branch: master',
-             'user: pingou changed poc of package: guake from: pingou to: '
-             'ralph on branch: el6'])
+            ['user: pingou changed point of contact of package: guake from: '
+             'orphan to: ralph on branch: master',
+             'user: pingou changed point of contact of package: guake from: '
+             'pingou to: ralph on branch: el6'])
 
         out = self.pkgdb.update_package_poc(
             'guake', ['master', 'el6'], 'pingou')
@@ -466,10 +491,10 @@ class TestPkgdDBAuth(unittest.TestCase):
         self.assertEqual(out['output'], 'ok')
         self.assertEqual(
             out['messages'],
-            ['user: pingou changed poc of package: guake from: ralph to: '
-             'pingou on branch: master',
-             'user: pingou changed poc of package: guake from: ralph to: '
-             'pingou on branch: el6'])
+            ['user: pingou changed point of contact of package: guake from: '
+             'ralph to: pingou on branch: master',
+             'user: pingou changed point of contact of package: guake from: '
+             'ralph to: pingou on branch: el6'])
 
 
 def suite():
